# Generated by Django 2.0.4 on 2018-11-01 20:50

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('slackchat', '0006_auto_20181101_2045'),
    ]

    operations = [
        migrations.AddField(
            model_name='message',
            name='serialized',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, null=True),
        ),
    ]



from collections import OrderedDict


class NoNonNullMixin(object):
    """Removes fields that are empty or null."""
    def to_representation(self, instance):
        representation = super().to_representation(instance)
        retained = filter(lambda x: x[1], representation.items())
        return OrderedDict(retained)



# flake8: noqa
from slackchat.tasks.channel import create_private_channel
from slackchat.tasks.user import update_users
from slackchat.tasks.webhook import (post_webhook, post_webhook_republish,
                                     verify_webhook)



import re

from django.contrib.postgres.fields import JSONField
from django.db import models
from django.utils.safestring import mark_safe
from markdown import markdown

from .custom_content_template import CustomContentTemplate


class Message(models.Model):
    """
    A message posted in a Slack channel by a user.
    """

    timestamp = models.DateTimeField(unique=True)

    channel = models.ForeignKey(
        "Channel", related_name="messages", on_delete=models.CASCADE
    )
    user = models.ForeignKey(
        "User", related_name="messages", on_delete=models.CASCADE
    )
    text = models.TextField()

    serialized = JSONField(blank=True, null=True)

    def html(self):
        return mark_safe(markdown(self.get_content()))

    def get_content(self):
        template, match = self.find_template_match()
        if template and template.content_template != "" and match:
            groups = [group for group in match.groups()]
            return template.content_template.format(*groups)
        return self.text

    def get_custom_args(self):
        template, match = self.find_template_match()
        if match:
            groups = [group for group in match.groups()]
            return [
                str(arg).strip().format(*groups)
                for arg in template.argument_template.split(",")
            ]
        return None

    def get_custom_kwargs(self):
        return self.parse_json_template("kwarg_template")

    def get_custom_attachment(self):
        return self.parse_json_template("attachment_template")

    def parse_json_template(self, template_type):
        template_schema, match = self.find_template_match()
        if match:
            groups = [group for group in match.groups()]
            output_dict = dict()
            template = getattr(template_schema, template_type)
            if isinstance(template, dict):
                for key in template:
                    template_key = getattr(template_schema, template_type)[key]
                    if isinstance(template_key, str):
                        output_dict[key] = template_key.format(*groups)
                    else:
                        output_dict[key] = template_key
                return output_dict
        return None

    def find_template_match(self):
        for template in CustomContentTemplate.objects.filter(
            chat_type=self.channel.chat_type
        ):
            match = re.search(template.search_string, self.text)
            if match:
                return (template, match)
        return (None, None)

    def serialize(self):
        self.save()

    def save(self, *args, **kwargs):
        from slackchat.serializers import MessageSerializer

        self.serialized = MessageSerializer(self).data
        super().save(*args, **kwargs)

    def __str__(self):
        return self.text[:50]



from django.db import models


class KeywordArgument(models.Model):
    """
    Replies to messages may contain key: value data.

    Used to create special handling for a message,
    e.g., tagging messages by category.
    """

    timestamp = models.DateTimeField(unique=True)
    key = models.SlugField(max_length=30)
    value = models.TextField()
    message = models.ForeignKey(
        'Message', on_delete=models.CASCADE, related_name='kwargs')
    user = models.ForeignKey('User', on_delete=models.CASCADE)

    def __str__(self):
        return '{0}: {1}'.format(self.key, self.value)



# Generated by Django 2.0.2 on 2018-05-07 13:15

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('slackchat', '0002_auto_20180414_2144'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='argument',
            unique_together={('character', 'chat_type')},
        ),
    ]



# Generated by Django 2.0.4 on 2018-11-01 20:45

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('slackchat', '0005_auto_20181015_2207'),
    ]

    operations = [
        migrations.AlterField(
            model_name='customcontenttemplate',
            name='attachment_template',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, help_text="A JSON object added to a message's attachments whose         values may be Python formatted strings. The args given to formatted         strings are the capture groups matched by the search string.", null=True),
        ),
        migrations.AlterField(
            model_name='customcontenttemplate',
            name='kwarg_template',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, help_text="A JSON object added to a message's kwargs whose values may         be Python formatted strings. The args given to formatted strings are         the capture groups matched by the search string.", null=True),
        ),
    ]



from django.db import transaction
from django.db.models.signals import post_delete, post_save
from django.dispatch import receiver
from slackchat.serializers import MessageSerializer

from .celery import (create_private_channel, post_webhook, update_users,
                     verify_webhook)
from .models import (Attachment, Channel, KeywordArgument, Message, Reaction,
                     User, Webhook)


@receiver(post_save, sender=Channel)
def save_channel(sender, instance, created, **kwargs):
    if created:
        create_private_channel.delay(instance.pk)


@receiver(post_save, sender=Webhook)
def save_webhook(sender, instance, **kwargs):
    verify_webhook.delay(instance.pk)


@receiver(post_save, sender=User)
def new_user(sender, instance, created, **kwargs):
    if created:
        update_users.delay([instance.pk])


@receiver(post_save, sender=Attachment)
@receiver(post_delete, sender=Attachment)
@receiver(post_save, sender=Reaction)
@receiver(post_delete, sender=Reaction)
@receiver(post_save, sender=KeywordArgument)
@receiver(post_delete, sender=KeywordArgument)
def reserialize_message(sender, instance, **kwargs):
    instance.message.serialize()


@receiver(post_save, sender=Message)
def notify_webhook(sender, instance, created, **kwargs):
    channel_id = instance.channel.id.hex
    message = MessageSerializer(instance=instance).data
    chat_type = instance.channel.chat_type.name

    if created:
        update_type = "message_created"
    else:
        update_type = "message_changed"

    transaction.on_commit(
        lambda: post_webhook.delay(channel_id, chat_type, update_type, message)
    )


@receiver(post_delete, sender=Message)
def notify_webhook_message_delete(sender, instance, **kwargs):
    channel_id = instance.channel.id.hex
    message = MessageSerializer(instance=instance).data
    chat_type = instance.channel.chat_type.name
    update_type = "message_deleted"

    transaction.on_commit(
        lambda: post_webhook.delay(channel_id, chat_type, update_type, message)
    )



from slackchat.models import Attachment, Message


def handle(message_pk, attachment):
    # TODO: For now, we're not attaching unfurled tweets.
    # This gets handled by markslack.
    if attachment.get('service_name') == 'twitter':
        return
    data = {
        'title': attachment.get('title'),
        'title_link': attachment.get('title_link'),
        'text': attachment.get('text'),
        'service_name': attachment.get('service_name'),
        'service_icon': attachment.get('service_icon'),
        'service_url': attachment.get('service_url'),
        'image_url': attachment.get('image_url'),
        'image_width': attachment.get('image_width'),
        'image_height': attachment.get('image_height'),
        'video_html': attachment.get('video_html'),
        'video_html_width': attachment.get('video_html_width'),
        'video_html_height': attachment.get('video_html_height'),
        'thumb_url': attachment.get('thumb_url'),
        'thumb_width': attachment.get('thumb_width'),
        'thumb_height': attachment.get('thumb_height'),
    }
    if data.get('title_link') or data.get('image_url'):
        message = Message.objects.get(pk=message_pk)
        Attachment.objects.get_or_create(message=message, **data)



# import json

# from django.conf import settings as project_settings
from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView
from slackchat.conf import settings

from .handlers import (
    handle_message,
    handle_message_removed,
    handle_reaction_added,
    handle_reaction_removed,
)


class Events(APIView):
    # Open API
    authentication_classes = ()
    permission_classes = ()

    def post(self, request, *args, **kwargs):
        slack_message = request.data

        # import json
        #
        # print("DEBUG slackchat log:")
        # print(json.dumps(slack_message, indent=2))

        if slack_message.get("token") != settings.SLACK_VERIFICATION_TOKEN:
            return Response(status=status.HTTP_403_FORBIDDEN)

        if slack_message.get("type") == "url_verification":
            return Response(
                data=slack_message.get("challenge"), status=status.HTTP_200_OK
            )

        if "event" in slack_message:
            id = slack_message.get("event_id")
            event = slack_message.get("event")

            if event.get("type") == "message":
                if event.get("subtype", None) == "message_deleted":
                    handle_message_removed(id, event)
                elif (
                    event.get("subtype", None) == "message_changed"
                    and event.get("message", {}).get("subtype", None)
                    == "tombstone"
                ):
                    handle_message_removed(id, event)
                else:
                    handle_message(id, event)

            if event.get("type") == "reaction_added":
                handle_reaction_added(id, event)

            if event.get("type") == "reaction_removed":
                handle_reaction_removed(id, event)

        return Response(status=status.HTTP_200_OK)



# flake8: noqa
from .attachment import AttachmentSerializer
from .channel import ChannelSerializer, ChannelListSerializer
from .chat_type import ChatTypeSerializer
from .message import MessageSerializer
from .reaction import ReactionSerializer
from .user import UserSerializer



from django.db import models


class Argument(models.Model):
    """
    A type of Reaction on a message that occurs
    when a specific emoji is applied.

    Actions are defined per ChatType.
    """
    name = models.SlugField(
        max_length=255,
        help_text="The name of the argument that will be attached to a \
         message.")
    character = models.CharField(max_length=100)
    chat_type = models.ForeignKey(
        'ChatType', on_delete=models.CASCADE)

    def __str__(self):
        return ':{0}: = {1} for chat type {2}'.format(
            self.character,
            self.name,
            self.chat_type
        )

    class Meta:
        unique_together = ('character', 'chat_type')



import uuid

import requests

from celery import shared_task
from django.core.files.base import ContentFile
from slackchat.conf import settings
from slackchat.models import User
from slacker import Slacker

SLACK = Slacker(settings.SLACK_API_TOKEN)


def get_slack_profile(user, users):
    """Return user profile from Slack."""
    for slack_user in users:
        if slack_user['id'] == user.api_id:
            return slack_user['profile']
    return None


@shared_task(acks_late=True)
def update_users(pks):
    users = SLACK.users.list().body['members']
    for pk in pks:
        user = User.objects.get(pk=pk)
        profile = get_slack_profile(user, users)

        real_name = profile.get('real_name', None)
        display_name = profile.get('display_name', None)
        try:
            first_name, last_name = real_name.split(' ', 1)
        except ValueError:
            try:
                first_name, last_name = display_name.split(' ', 1)
            except ValueError:
                first_name = real_name or display_name or ''
                last_name = ''

        user.first_name = first_name
        user.last_name = last_name
        user.title = profile.get('title', 'Staff writer')
        user.email = profile.get('email', None)

        if profile.get('image_192', False):
            r = requests.get(profile.get('image_192'), stream=True)
            img = r.raw.read()
            user.image.save(
                'profile-{}.jpg'.format(uuid.uuid4().hex[:10]),
                ContentFile(img),
                save=True
            )

        user.save()



from rest_framework import serializers
from slackchat.models import Reaction


class ReactionSerializer(serializers.ModelSerializer):
    user = serializers.SerializerMethodField()

    def get_user(self, obj):
        return obj.user.api_id

    class Meta:
        model = Reaction
        fields = (
            'timestamp',
            'reaction',
            'user'
        )



import os

from django.utils import timezone
from rest_framework import serializers
from slackchat.models import Channel

from .message import MessageSerializer
from .user import UserSerializer


class ChannelSerializer(serializers.ModelSerializer):
    chat_type = serializers.SerializerMethodField()
    publish_path = serializers.SerializerMethodField()
    paths = serializers.SerializerMethodField()
    users = serializers.SerializerMethodField()
    meta = serializers.SerializerMethodField()
    introduction = serializers.SerializerMethodField()
    timestamp = serializers.SerializerMethodField()
    messages = serializers.SerializerMethodField()

    def get_chat_type(self, obj):
        return obj.chat_type.name

    def get_paths(self, obj):
        return {
            "channel": obj.publish_path,
            "chat_type": obj.chat_type.publish_path,
        }

    def get_publish_path(self, obj):
        """
        publish_path joins the publish_paths for the chat type and the channel.
        """
        return os.path.join(
            obj.chat_type.publish_path, obj.publish_path.lstrip("/")
        )

    def get_users(self, obj):
        users = {}

        for message in obj.messages.all():
            user = message.user
            serializer = UserSerializer(instance=user)
            if user.api_id not in users:
                users[user.api_id] = serializer.data

            for reaction in message.reactions.all():
                serializer = UserSerializer(instance=reaction.user)
                if reaction.user.api_id not in users:
                    users[reaction.user.api_id] = serializer.data

        return users

    def get_timestamp(self, obj):
        return timezone.now()

    def get_meta(self, obj):
        return {
            "title": obj.meta_title,
            "description": obj.meta_description,
            "image": obj.meta_image,
        }

    def get_introduction(self, obj):
        return obj.get_introduction()

    def get_messages(self, obj):
        return [m.serialized for m in obj.messages.all()]

    class Meta:
        model = Channel
        fields = (
            "id",
            "api_id",
            "chat_type",
            "title",
            "introduction",
            "meta",
            "extras",
            "paths",
            "publish_path",
            "publish_time",
            "live",
            "users",
            "messages",
            "timestamp",
        )


class ChannelListSerializer(serializers.ModelSerializer):
    chat_type = serializers.SerializerMethodField()

    def get_chat_type(self, obj):
        return obj.chat_type.name

    class Meta:
        model = Channel
        fields = ("id", "api_id", "chat_type")



from django.db import models
from django.contrib.postgres.fields import JSONField


class CustomContentTemplate(models.Model):
    """
    Defines search parameters for finding custom messages
    and a template for how to serialize the message.
    """

    name = models.CharField(max_length=255)
    search_string = models.CharField(
        max_length=255, help_text="A regex search string with capture groups."
    )
    chat_type = models.ForeignKey("ChatType", on_delete=models.CASCADE)
    content_template = models.TextField(
        blank=True,
        null=True,
        help_text="A Python format string whose args are the capture groups \
        matched by the search_string.",
    )
    argument_template = models.CharField(
        blank=True,
        null=True,
        max_length=255,
        help_text="A comma seprated list of Python format string whose \
        args are the capture groups matched by the search_string.",
    )
    attachment_template = JSONField(
        blank=True,
        null=True,
        help_text="A JSON object added to a message's attachments whose \
        values may be Python formatted strings. The args given to formatted \
        strings are the capture groups matched by the search string.",
    )
    kwarg_template = JSONField(
        blank=True,
        null=True,
        help_text="A JSON object added to a message's kwargs whose values may \
        be Python formatted strings. The args given to formatted strings are \
        the capture groups matched by the search string.",
    )

    def __str__(self):
        return self.name



# Generated by Django 2.0.4 on 2018-10-15 20:32

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('slackchat', '0003_auto_20180507_1315'),
    ]

    operations = [
        migrations.AddField(
            model_name='customcontenttemplate',
            name='argument_template',
            field=models.TextField(blank=True, help_text='A Python format string whose args are the capture groups         matched by the search_string.', null=True),
        ),
        migrations.AddField(
            model_name='customcontenttemplate',
            name='attachment_template',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, help_text="A JSON object with Python formatted strings whose args are         the capture groups matched by the search_string and added to the         message's attachments.", null=True),
        ),
        migrations.AddField(
            model_name='customcontenttemplate',
            name='kwarg_template',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, help_text="A JSON object with Python formatted strings whose args are         the capture groups matched by the search_string and added to the         message's kwargs (overridden by manual kwargs in the thread).", null=True),
        ),
        migrations.AlterField(
            model_name='customcontenttemplate',
            name='content_template',
            field=models.TextField(blank=True, help_text='A Python format string whose args are the capture groups         matched by the search_string.', null=True),
        ),
    ]



from django.conf import settings as project_settings


class Settings:
    pass


Settings.SLACK_VERIFICATION_TOKEN = getattr(
    project_settings,
    'SLACKCHAT_SLACK_VERIFICATION_TOKEN',
    None
)

Settings.SLACK_API_TOKEN = getattr(
    project_settings,
    'SLACKCHAT_SLACK_API_TOKEN',
    None
)

Settings.WEBHOOK_VERIFICATION_TOKEN = getattr(
    project_settings,
    'SLACK_WEBHOOK_VERIFICATION_TOKEN',
    'slackchat',
)

Settings.PUBLISH_ROOT = getattr(
    project_settings,
    'SLACKCHAT_PUBLISH_ROOT',
    None,
)

Settings.MARKSLACK_USER_TEMPLATE = getattr(
    project_settings,
    'SLACK_MARKSLACK_USER_TEMPLATE',
    lambda user: '<span class="mention">{} {}</span>'.format(
        user.first_name,
        user.last_name
    ),
)

Settings.MARKSLACK_LINK_TEMPLATES = getattr(
    project_settings,
    'SLACK_MARKSLACK_LINK_TEMPLATES',
    {
        'twitter.com': '<blockquote class="twitter-tweet" data-lang="en"><a href="{}"></a></blockquote>', # noqa
    },
)

Settings.MARKSLACK_IMAGE_TEMPLATE = getattr(
    project_settings,
    'SLACK_MARKSLACK_IMAGE_TEMPLATE',
    '[(image)]({})',
)


def default_user_image_upload_to(instance, filename):
    return 'slackchat/users/{0}{1}/{2}'.format(
        instance.first_name,
        instance.last_name,
        filename,
    )


Settings.USER_IMAGE_UPLOAD_TO = getattr(
    project_settings,
    'SLACKCHAT_USER_IMAGE_UPLOAD_TO',
    default_user_image_upload_to
)


settings = Settings



import os
import uuid
from datetime import datetime
from urllib.parse import urljoin

from django.contrib.postgres.fields import JSONField
from django.db import models
from django.urls import reverse
from django.utils import timezone
from django.utils.encoding import escape_uri_path
from django.utils.safestring import mark_safe
from markdown import markdown
from slackchat.conf import settings
from slackchat.fields import MarkdownField


class Channel(models.Model):
    """
    A Slack channel that hosts a slackchat.
    """
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)

    api_id = models.SlugField(
        max_length=10, null=True, blank=True, editable=False,
        help_text="Slack API channel ID"
    )

    team_id = models.SlugField(
        max_length=10, null=True, blank=True, editable=False,
        help_text="Slack API team ID"
    )

    chat_type = models.ForeignKey(
        'ChatType', on_delete=models.PROTECT)

    owner = models.ForeignKey('slackchat.User', on_delete=models.PROTECT)

    title = models.CharField(max_length=150, blank=True, null=True)

    introduction = MarkdownField(
        blank=True, null=True,
        help_text="Some introductory paragraph text (in markdown syntax).")

    meta_title = models.CharField(
        max_length=300, blank=True, null=True,
        help_text="Page title")
    meta_description = models.CharField(
        max_length=300, blank=True, null=True,
        help_text="Page description")
    meta_image = models.URLField(
        blank=True, null=True,
        help_text="Share image URL")

    # Extra metadata, provided by django-foreignform
    extras = JSONField(blank=True, null=True)

    publish_path = models.CharField(
        max_length=300, blank=True, unique=True,
        help_text="A relative path you can use when \
        publishing the slackchat, e.g., \
        <span style='color:grey; font-weight:bold;'>/2018-02-12/econ-chat/\
        </span>."
    )

    publish_time = models.DateTimeField(
        default=timezone.now, help_text="Dateline.")

    live = models.BooleanField(
        default=False, help_text="Determines whether page should re-poll for new messages \
        while chat is live.")

    @property
    def published_link(self):
        if settings.PUBLISH_ROOT:
            relative_path = os.path.join(
                self.chat_type.publish_path,
                self.publish_path.lstrip('/')
            ).lstrip('/')
            link = urljoin(settings.PUBLISH_ROOT, relative_path)
            return mark_safe(
                '<a href="{0}" target="_blank">{0}</a>'.format(link))
        else:
            return os.path.join(
                self.chat_type.publish_path,
                self.publish_path.lstrip('/')
            )

    @property
    def api_link(self):
        link = reverse('slackchat-channel-detail', args=[self.id])
        return mark_safe(
            '<a href="{0}" target="_blank">{0}</a>'.format(link))

    @property
    def slack_link(self):
        if self.team_id:
            return mark_safe(
                '<a href="slack://channel?id={0}&team={1}"\
                 target="_blank">{2}</a>'.format(
                    self.api_id,
                    self.team_id,
                    self.slackchat
                ))
        else:
            return '-'

    @property
    def slackchat(self):
        return 'slackchat-{}'.format(self.id.hex[:10])

    def save(self, *args, **kwargs):
        if self.publish_path:
            self.publish_path = escape_uri_path(self.publish_path)
        else:
            self.publish_path = '/{}/{}/'.format(
                datetime.today().strftime('%Y-%m-%d'),
                self.id.hex[:8]
            )
        super().save(*args, **kwargs)

    def get_introduction(self):
        if self.chat_type.render_to_html:
            return mark_safe(markdown(self.introduction))
        return self.introduction

    def __str__(self):
        return self.slackchat



# Generated by Django 2.0.4 on 2018-10-15 22:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('slackchat', '0004_auto_20181015_2032'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='customcontenttemplate',
            name='argument_name',
        ),
        migrations.AlterField(
            model_name='customcontenttemplate',
            name='argument_template',
            field=models.CharField(blank=True, help_text='A comma seprated list of Python format string whose         args are the capture groups matched by the search_string.', max_length=255, null=True),
        ),
    ]



from django.contrib import admin
from foreignform.mixins import ForeignFormAdminMixin

from .celery import post_webhook, post_webhook_republish, update_users
from .models import (Argument, Attachment, Channel, ChatType,
                     CustomContentTemplate, KeywordArgument, Message, Reaction,
                     User, Webhook)


class ChatTypeAdmin(admin.ModelAdmin):
    fieldsets = (
        (None, {
            'fields': (
                'name',
                'publish_path',
            ),
        }),
        ('Channel options', {
            'fields': (
                'render_to_html',
                'kwargs_in_threads',
            ),
        }),
        ('Extra channel fields', {
            'fields': (
                'jsonSchema',
                'uiSchema',
            ),
            'description': (
                'See the documentation for <a href="'
                'https://github.com/mozilla-services/react-jsonschema-form"'
                ' target="_blank"> '
                'react-jsonschema-form</a> to learn how to configure extra '
                'fields for channels of this type.'
            ),
        }),
    )


class ChannelAdmin(ForeignFormAdminMixin, admin.ModelAdmin):
    foreignform_foreign_key = 'chat_type'
    foreignform_field = 'extras'
    readonly_fields = ('api_id', 'team_id',)
    fieldsets = (
        (None, {
            'fields': ('api_id', 'team_id', 'chat_type', 'owner')
        }),
        ('Publishing info', {
            'fields': ('publish_path', 'publish_time', 'live')
        }),
        ('Page content', {
            'fields': ('title', 'introduction')
        }),
        ('SEO tags', {
            'fields': ('meta_title', 'meta_description', 'meta_image')
        }),
        ('Extra channel fields', {
            'fields': ('extras',),
        }),
    )
    list_display = (
        'slackchat',
        'published_link',
        'api_link',
        'slack_link',
        'live',
    )
    autocomplete_fields = ['owner']
    actions = [
        'request_chats_update',
        'request_chats_republish',
        'close_live_chats',
    ]

    def request_chats_update(self, request, queryset):
        for channel in queryset:
            post_webhook.delay(channel.id.hex, channel.chat_type.name)
        self.message_user(request, 'Requested channels update chat page!')

    def request_chats_republish(self, request, queryset):
        for channel in queryset:
            post_webhook_republish.delay(
                channel.id.hex, channel.chat_type.name)
        self.message_user(request, 'Requested channels republish chat page!')

    def close_live_chats(self, request, queryset):
        queryset.update(live=False)
        for channel in queryset:
            post_webhook.delay(channel.id.hex, channel.chat_type.name)
        self.message_user(
            request,
            'Closed live channels and requested they update chat page!')


class UserAdmin(admin.ModelAdmin):
    search_fields = ['first_name', 'last_name']
    actions = ['update_user_profiles']

    def update_user_profiles(self, request, queryset):
        update_users.delay([user.pk for user in queryset])
        self.message_user(request, 'Updated user profiles!')


class WebhookAdmin(admin.ModelAdmin):
    fields = ('endpoint', 'verified',)


admin.site.register(Argument)
admin.site.register(Attachment)
admin.site.register(Channel, ChannelAdmin)
admin.site.register(User, UserAdmin)
admin.site.register(CustomContentTemplate)
admin.site.register(Message)
admin.site.register(Reaction)
admin.site.register(KeywordArgument)
admin.site.register(ChatType, ChatTypeAdmin)
admin.site.register(Webhook, WebhookAdmin)



from django.db import models


class Attachment(models.Model):
    """
    An attachment to a Slack message.
    """
    message = models.ForeignKey(
        'Message', related_name='attachments', on_delete=models.CASCADE)

    title = models.CharField(max_length=300, blank=True, null=True)
    title_link = models.URLField(blank=True, null=True)
    text = models.TextField(blank=True, null=True)

    service_name = models.CharField(max_length=300, blank=True, null=True)
    service_icon = models.URLField(blank=True, null=True)
    service_url = models.URLField(blank=True, null=True)

    image_url = models.URLField(blank=True, null=True)
    image_width = models.PositiveSmallIntegerField(blank=True, null=True)
    image_height = models.PositiveSmallIntegerField(blank=True, null=True)

    video_html = models.TextField(blank=True, null=True)
    video_html_width = models.PositiveSmallIntegerField(blank=True, null=True)
    video_html_height = models.PositiveSmallIntegerField(blank=True, null=True)

    thumb_url = models.URLField(blank=True, null=True)
    thumb_width = models.PositiveSmallIntegerField(blank=True, null=True)
    thumb_height = models.PositiveSmallIntegerField(blank=True, null=True)

    def __str__(self):
        return self.title_link or self.image_url

    class Meta:
        unique_together = (
            ('message', 'title_link', 'image_url'),
        )
